@@ -12,8 +12,10 @@
 #include <string.h>
 #include <math.h>
 
+#ifndef _MSC_VER
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
+#endif
 
 #ifdef OPENGL
 #include <GL/gl.h>
@@ -44,6 +46,58 @@ extern Display  *dpy;
 extern Globaldata xobjs;
 extern Clientdata areastruct;
 
+/*----------------------------------------------------------------------*/
+/* Recursive search for graphic images in an object.			*/
+/* Updates list "glist" when any image is found in an object or its	*/
+/* descendents.								*/
+/*----------------------------------------------------------------------*/
+
+void count_graphics(objectptr thisobj, short *glist)
+{
+   genericptr *ge;
+   graphicptr gp;
+   Imagedata *iptr;
+   int i;
+
+   for (ge = thisobj->plist; ge < thisobj->plist + thisobj->parts; ge++) {
+      if (IS_GRAPHIC(*ge)) {
+	 gp = TOGRAPHIC(ge);
+         for (i = 0; i < xobjs.images; i++) {
+	    iptr = xobjs.imagelist + i;
+            if (iptr->image == gp->source) {
+	       glist[i]++;
+	    }
+	 }
+      }
+      else if (IS_OBJINST(*ge)) {
+	 count_graphics(TOOBJINST(ge)->thisobject, glist);
+      }
+   }
+}
+
+/*----------------------------------------------------------------------*/
+/* Given a list of pages, return a list of indices into the graphics	*/
+/* buffer area of each graphic used on any of the indicated pages.	*/
+/* The returned list is allocated and it is the responsibility of the	*/
+/* calling routine to free it.						*/
+/*----------------------------------------------------------------------*/
+
+short *collect_graphics(short *pagelist)
+{
+   short *glist;
+   int i;
+
+   glist = (short *)malloc(xobjs.images * sizeof(short));
+
+   for (i = 0; i < xobjs.images; i++) glist[i] = 0;
+
+   for (i = 0; i < xobjs.pages; i++)
+      if (pagelist[i] > 0)
+	 count_graphics(xobjs.pagelist[i]->pageinst->thisobject, glist);
+	
+   return glist;
+}
+
 /*----------------------------------------------------------------------*/
 /* Declare all graphic image targets to be invalid.   This is required	*/
 /* on any change in the zoom factor of the current object.		*/
@@ -221,10 +275,10 @@ graphicptr new_graphic(objinstptr destinst, char *filename, int px, int py)
     Imagedata *iptr;
     FILE *fg;
     int nf, nr, width, height, imax, x, y, i;
-    char id[5], c;
+    char id[5], c, buf[128];
     union {
-       unsigned char b[4];
-       unsigned long i;
+       u_char b[4];
+       u_long i;
     } pixel;
 
     locdestinst = (destinst == NULL) ? areastruct.topinstance : destinst;
@@ -248,12 +302,53 @@ graphicptr new_graphic(objinstptr destinst, char *filename, int px, int py)
        /* This ONLY handles binary ppm files with max data = 255 */
 
        while (1) {
-          nr = fscanf(fg, "%4s %d %d %d", id, &width, &height, &imax);
-          if ((nr == 4) && (id[0] != '#')) break;
-          else if (id[0] != '#') return NULL; 
+          nr = fscanf(fg, " %s", buf);
+	  if (nr <= 0) return NULL;
+	  if (buf[0] != '#') {
+	     if (sscanf(buf, "%s", id) <= 0)
+		return NULL;
+	     break;
+	  }
+	  else fgets(buf, 127, fg);
+       }
+       if ((nr <= 0) || strncmp(id, "P6", 2)) return NULL;
+
+       while (1) {
+          nr = fscanf(fg, " %s", buf);
+	  if (nr <= 0) return NULL;
+	  if (buf[0] != '#') {
+	     if (sscanf(buf, "%d", &width) <= 0)
+		return NULL;
+	     break;
+	  }
+	  else fgets(buf, 127, fg);
+       }
+       if (width <= 0) return NULL;
+
+       while (1) {
+          nr = fscanf(fg, " %s", buf);
+	  if (nr <= 0) return NULL;
+	  if (buf[0] != '#') {
+	     if (sscanf(buf, "%d", &height) <= 0)
+		return NULL;
+	     break;
+	  }
+	  else fgets(buf, 127, fg);
+       }
+       if (height <= 0) return NULL;
+
+       while (1) {
+          nr = fscanf(fg, " %s", buf);
+	  if (nr <= 0) return NULL;
+	  if (buf[0] != '#') {
+	     if (sscanf(buf, "%d", &imax) <= 0)
+		return NULL;
+	     break;
+	  }
+	  else fgets(buf, 127, fg);
        }
-       if (strncmp(id, "P6", 2)) return NULL;
        if (imax != 255) return NULL;
+
        while (1) {
           fread(&c, 1, 1, fg);
           if (c == '\n') break;
